@@ -1,26 +1,38 @@
 #pragma once
 
 #include <list>
-#include <vector>
 #include <map>
+#include <vector>
 
-#include "zeek/Obj.h"
+#include "zeek/ID.h"
+#include "zeek/IntrusivePtr.h"
 #include "zeek/Notifier.h"
+#include "zeek/Obj.h"
 #include "zeek/iosource/IOSource.h"
 #include "zeek/util.h"
-#include "zeek/IntrusivePtr.h"
 
-ZEEK_FORWARD_DECLARE_NAMESPACED(ODesc, zeek);
-ZEEK_FORWARD_DECLARE_NAMESPACED(Val, zeek);
-ZEEK_FORWARD_DECLARE_NAMESPACED(Frame, zeek::detail);
-ZEEK_FORWARD_DECLARE_NAMESPACED(Stmt, zeek::detail);
-ZEEK_FORWARD_DECLARE_NAMESPACED(Expr, zeek::detail);
-ZEEK_FORWARD_DECLARE_NAMESPACED(CallExpr, zeek::detail);
-ZEEK_FORWARD_DECLARE_NAMESPACED(ID, zeek::detail);
+namespace zeek
+	{
 
-namespace zeek::detail {
-namespace trigger {
+class ODesc;
+class Val;
 
+using ValPtr = IntrusivePtr<Val>;
+
+namespace detail
+	{
+
+class Frame;
+class Stmt;
+class Expr;
+class CallExpr;
+class ID;
+class WhenInfo;
+
+using StmtPtr = IntrusivePtr<Stmt>;
+
+namespace trigger
+	{
 
 // Triggers are the heart of "when" statements: expressions that when
 // they become true execute a body of statements.
@@ -28,14 +40,22 @@ namespace trigger {
 class TriggerTimer;
 class TriggerTraversalCallback;
 
-class Trigger final : public Obj, public notifier::detail::Receiver {
+class Trigger final : public Obj, public notifier::detail::Receiver
+	{
 public:
 	// Don't access Trigger objects; they take care of themselves after
 	// instantiation.  Note that if the condition is already true, the
 	// statements are executed immediately and the object is deleted
 	// right away.
-	Trigger(Expr* cond, Stmt* body, Stmt* timeout_stmts, Expr* timeout,
-		Frame* f, bool is_return, const Location* loc);
+
+	// These first constructor is for the deprecated deep-copy semantics.
+	Trigger(ExprPtr cond, StmtPtr body, StmtPtr timeout_stmts, double timeout, Frame* f,
+	        bool is_return, const Location* loc);
+
+	// Used for capture-list semantics.
+	Trigger(WhenInfo* wi, double timeout, const IDSet& globals, std::vector<ValPtr> local_aggrs,
+	        Frame* f, const Location* loc);
+
 	~Trigger() override;
 
 	// Evaluates the condition. If true, executes the body and deletes
@@ -48,15 +68,14 @@ public:
 	void Timeout();
 
 	// Return the timeout interval (negative if none was specified).
-	double TimeoutValue() const
-		{ return timeout_value; }
+	double TimeoutValue() const { return timeout_value; }
 
 	// Called if another entity needs to complete its operations first
 	// in any case before this trigger can proceed.
-	void Hold()	{ delayed = true; }
+	void Hold() { delayed = true; }
 
 	// Complement of Hold().
-	void Release()	{ delayed = false; }
+	void Release() { delayed = false; }
 
 	// If we evaluate to true, our return value will be passed on
 	// to the given trigger.  Note, automatically calls Hold().
@@ -64,9 +83,24 @@ public:
 
 	// Cache for return values of delayed function calls.  Returns whether
 	// the trigger is queued for later evaluation -- it may not be queued
-	// if the Val is null or it's disabled.
-	bool Cache(const CallExpr* expr, Val* val);
-	Val* Lookup(const CallExpr*);
+	// if the Val is null or it's disabled.  The cache is managed using
+	// void*'s so that the value can be associated with either a CallExpr
+	// (for interpreted execution) or a C++ function (for compiled-to-C++).
+	bool Cache(const void* obj, Val* val);
+	Val* Lookup(const void* obj);
+
+	[[deprecated(
+		"Remove in v5.1. Use Frame::GetTriggerAssoc() / const void* interface instead.")]] bool
+	Cache(const CallExpr* call, Val* val)
+		{
+		return Cache((const void*)call, val);
+		}
+	[[deprecated(
+		"Remove in v5.1. Use Frame::GetTriggerAssoc() / const void* interface instead.")]] Val*
+	Lookup(const CallExpr* call)
+		{
+		return Lookup((const void*)call);
+		}
 
 	// Disable this trigger completely. Needed because Unref'ing the trigger
 	// may not immediately delete it as other references may still exist.
@@ -85,25 +119,30 @@ public:
 	// for any further progress to be made, so just Unref ourselves.
 	void Terminate() override;
 
-	const char* Name() const;
+	const char* Name() const { return name.c_str(); }
+	void SetName(const char* new_name) { name = new_name; }
 
 private:
-	friend class TriggerTraversalCallback;
 	friend class TriggerTimer;
 
-	void Init(std::vector<IntrusivePtr<Val>> index_expr_results);
-	void Register(ID* id);
+	void Init(ExprPtr cond, StmtPtr body, StmtPtr timeout_stmts, Frame* frame, bool is_return,
+	          const Location* location);
+
+	void ReInit(std::vector<ValPtr> index_expr_results);
+
+	void Register(const ID* id);
 	void Register(Val* val);
 	void UnregisterAll();
 
-	Expr* cond;
-	Stmt* body;
-	Stmt* timeout_stmts;
-	Expr* timeout;
+	ExprPtr cond;
+	StmtPtr body;
+	StmtPtr timeout_stmts;
+	ExprPtr timeout;
 	double timeout_value;
 	Frame* frame;
 	bool is_return;
-	const Location* location;
+
+	std::string name;
 
 	TriggerTimer* timer;
 	Trigger* attached;
@@ -111,17 +150,28 @@ private:
 	bool delayed; // true if a function call is currently being delayed
 	bool disabled;
 
-	std::vector<std::pair<Obj *, notifier::detail::Modifiable*>> objs;
+	// Globals and locals present in the when expression.
+	IDSet globals;
+	IDSet locals; // not needed, present only for matching deprecated logic
 
-	using ValCache = std::map<const CallExpr*, Val*>;
+	// Tracks whether we've found the globals/locals, as the work only
+	// has to be done once.
+	bool have_trigger_elems = false;
+
+	// Aggregate values seen in locals used in the trigger condition,
+	// so we can detect changes in them that affect whether the condition
+	// holds.
+	std::vector<ValPtr> local_aggrs;
+
+	std::vector<std::pair<Obj*, notifier::detail::Modifiable*>> objs;
+
+	using ValCache = std::map<const void*, Val*>;
 	ValCache cache;
-};
+	};
 
-using TriggerPtr = IntrusivePtr<Trigger>;
-
-class Manager final : public iosource::IOSource {
+class Manager final : public iosource::IOSource
+	{
 public:
-
 	Manager();
 	~Manager();
 
@@ -131,31 +181,23 @@ public:
 
 	void Queue(Trigger* trigger);
 
-	struct Stats {
+	struct Stats
+		{
 		unsigned long total;
 		unsigned long pending;
-	};
+		};
 
 	void GetStats(Stats* stats);
 
 private:
-
 	using TriggerList = std::list<Trigger*>;
 	TriggerList* pending;
 	unsigned long total_triggers = 0;
 	};
 
-} // namespace trigger
+	} // namespace trigger
 
 extern trigger::Manager* trigger_mgr;
 
-} // namespace zeek::detail
-
-namespace trigger {
-
-using Trigger [[deprecated("Remove in v4.1. Use zeek::detail::trigger::Trigger.")]] = zeek::detail::trigger::Trigger;
-using Manager [[deprecated("Remove in v4.1. Use zeek::detail::trigger::Manager.")]] = zeek::detail::trigger::Manager;
-
-} // namespace trigger
-
-extern zeek::detail::trigger::Manager*& trigger_mgr [[deprecated("Remove in v4.1. Use zeek::detail::trigger_mgr.")]];
+	} // namespace detail
+	} // namespace zeek
