@@ -2,34 +2,41 @@
 
 #pragma once
 
-#include <string>
-#include <set>
 #include <map>
+#include <set>
+#include <string>
 
-#include "zeek/file_analysis/Component.h"
-#include "zeek/RunState.h"
 #include "zeek/RuleMatcher.h"
-
-#include "zeek/plugin/ComponentManager.h"
-#include "zeek/analyzer/Tag.h"
+#include "zeek/RunState.h"
+#include "zeek/Tag.h"
+#include "zeek/file_analysis/Component.h"
 #include "zeek/file_analysis/FileTimer.h"
+#include "zeek/plugin/ComponentManager.h"
 
-ZEEK_FORWARD_DECLARE_NAMESPACED(TableVal, zeek);
-ZEEK_FORWARD_DECLARE_NAMESPACED(VectorVal, zeek);
-ZEEK_FORWARD_DECLARE_NAMESPACED(Analyzer, zeek, analyzer);
-ZEEK_FORWARD_DECLARE_NAMESPACED(Tag, zeek, analyzer);
-ZEEK_FORWARD_DECLARE_NAMESPACED(File, zeek, file_analysis);
-ZEEK_FORWARD_DECLARE_NAMESPACED(Tag, zeek, file_analysis);
+namespace zeek
+	{
 
-namespace zeek {
-namespace file_analysis {
+class TableVal;
+class VectorVal;
+
+namespace analyzer
+	{
+
+class Analyzer;
+
+	} // namespace analyzer
+
+namespace file_analysis
+	{
+
+class File;
 
 /**
  * Main entry point for interacting with file analysis.
  */
-class Manager : public plugin::ComponentManager<Tag, Component> {
+class Manager : public plugin::ComponentManager<Component>
+	{
 public:
-
 	/**
 	 * Constructor.
 	 */
@@ -42,13 +49,13 @@ public:
 
 	/**
 	 * First-stage initializion of the manager. This is called early on
-	 * during Bro's initialization, before any scripts are processed.
+	 * during Zeek's initialization, before any scripts are processed.
 	 */
 	void InitPreScript();
 
 	/**
 	 * Second-stage initialization of the manager. This is called late
-	 * during Bro's initialization after any scripts are processed.
+	 * during Zeek's initialization after any scripts are processed.
 	 */
 	void InitPostScript();
 
@@ -103,9 +110,8 @@ public:
 	 *         the \c get_file_handle script-layer event).  An empty string
 	 *         indicates the associate file is not going to be analyzed further.
 	 */
-	std::string DataIn(const u_char* data, uint64_t len, uint64_t offset,
-	                   const analyzer::Tag& tag, Connection* conn, bool is_orig,
-	                   const std::string& precomputed_file_id = "",
+	std::string DataIn(const u_char* data, uint64_t len, uint64_t offset, const zeek::Tag& tag,
+	                   Connection* conn, bool is_orig, const std::string& precomputed_file_id = "",
 	                   const std::string& mime_type = "");
 
 	/**
@@ -130,9 +136,8 @@ public:
 	 *         the \c get_file_handle script-layer event).  An empty string
 	 *         indicates the associated file is not going to be analyzed further.
 	 */
-	std::string DataIn(const u_char* data, uint64_t len, const analyzer::Tag& tag,
-	                   Connection* conn, bool is_orig,
-	                   const std::string& precomputed_file_id = "",
+	std::string DataIn(const u_char* data, uint64_t len, const zeek::Tag& tag, Connection* conn,
+	                   bool is_orig, const std::string& precomputed_file_id = "",
 	                   const std::string& mime_type = "");
 
 	/**
@@ -143,9 +148,34 @@ public:
 	 * @param source uniquely identifies the file and should also describe
 	 *        in human-readable form where the file input is coming from (e.g.
 	 *        a local file path).
+	 * @param mime_type may be set to the mime type of the file, if already known due
+	 *        to the protocol. This is, e.g., the case in TLS connections where X.509
+	 *        certificates are passed as files; here the type of the file is set by
+	 *        the protocol. If this parameter is give, mime type detection will be
+	 *        disabled.
+	 *        This parameter is only used for the first bit of data for each file.
 	 */
 	void DataIn(const u_char* data, uint64_t len, const std::string& file_id,
-	            const std::string& source);
+	            const std::string& source, const std::string& mime_type = "");
+
+	/**
+	 * Pass in sequential file data from external source (e.g. input framework).
+	 * @param data pointer to start of a chunk of file data.
+	 * @param len number of bytes in the data chunk.
+	 * @param offset number of bytes from start of file that data chunk occurs.
+	 * @param file_id an identifier for the file (usually a hash of \a source).
+	 * @param source uniquely identifies the file and should also describe
+	 *        in human-readable form where the file input is coming from (e.g.
+	 *        a local file path).
+	 * @param mime_type may be set to the mime type of the file, if already known due
+	 *        to the protocol. This is, e.g., the case in TLS connections where X.509
+	 *        certificates are passed as files; here the type of the file is set by
+	 *        the protocol. If this parameter is give, mime type detection will be
+	 *        disabled.
+	 *        This parameter is only used for the first bit of data for each file.
+	 */
+	void DataIn(const u_char* data, uint64_t len, uint64_t offset, const std::string& file_id,
+	            const std::string& source, const std::string& mime_type = "");
 
 	/**
 	 * Signal the end of file data regardless of which direction it is being
@@ -153,7 +183,7 @@ public:
 	 * @param tag network protocol over which the file data is transferred.
 	 * @param conn network connection over which the file data is transferred.
 	 */
-	void EndOfFile(const analyzer::Tag& tag, Connection* conn);
+	void EndOfFile(const zeek::Tag& tag, Connection* conn);
 
 	/**
 	 * Signal the end of file data being transferred over a connection in
@@ -161,7 +191,7 @@ public:
 	 * @param tag network protocol over which the file data is transferred.
 	 * @param conn network connection over which the file data is transferred.
 	 */
-	void EndOfFile(const analyzer::Tag& tag, Connection* conn, bool is_orig);
+	void EndOfFile(const zeek::Tag& tag, Connection* conn, bool is_orig);
 
 	/**
 	 * Signal the end of file data being transferred using the file identifier.
@@ -185,9 +215,8 @@ public:
 	 *         the \c get_file_handle script-layer event).  An empty string
 	 *         indicates the associate file is not going to be analyzed further.
 	 */
-	std::string Gap(uint64_t offset, uint64_t len, const analyzer::Tag& tag,
-	                Connection* conn, bool is_orig,
-	                const std::string& precomputed_file_id = "");
+	std::string Gap(uint64_t offset, uint64_t len, const zeek::Tag& tag, Connection* conn,
+	                bool is_orig, const std::string& precomputed_file_id = "");
 
 	/**
 	 * Provide the expected number of bytes that comprise a file.
@@ -204,8 +233,8 @@ public:
 	 *         the \c get_file_handle script-layer event).  An empty string
 	 *         indicates the associate file is not going to be analyzed further.
 	 */
-	std::string SetSize(uint64_t size, const analyzer::Tag& tag, Connection* conn,
-	                    bool is_orig, const std::string& precomputed_file_id = "");
+	std::string SetSize(uint64_t size, const zeek::Tag& tag, Connection* conn, bool is_orig,
+	                    const std::string& precomputed_file_id = "");
 
 	/**
 	 * Starts ignoring a file, which will finally be removed from internal
@@ -250,12 +279,7 @@ public:
 	 * @return false if file identifier and analyzer did not map to anything,
 	 *         else true.
 	 */
-	bool SetExtractionLimit(const std::string& file_id,
-	                        RecordValPtr args, uint64_t n) const;
-
-	[[deprecated("Remove in v4.1.  Pass IntrusivePtr args param instead.")]]
-	bool SetExtractionLimit(const std::string& file_id, RecordVal* args,
-	                        uint64_t n) const;
+	bool SetExtractionLimit(const std::string& file_id, RecordValPtr args, uint64_t n) const;
 
 	/**
 	 * Try to retrieve a file that's being analyzed, using its identifier/hash.
@@ -274,12 +298,7 @@ public:
 	 * @param args a \c AnalyzerArgs value which describes a file analyzer.
 	 * @return false if the analyzer failed to be instantiated, else true.
 	 */
-	bool AddAnalyzer(const std::string& file_id, const file_analysis::Tag& tag,
-	                 RecordValPtr args) const;
-
-	[[deprecated("Remove in v4.1.  Pass IntrusivePtr args param instead.")]]
-	bool AddAnalyzer(const std::string& file_id, const file_analysis::Tag& tag,
-	                 RecordVal* args) const;
+	bool AddAnalyzer(const std::string& file_id, const zeek::Tag& tag, RecordValPtr args) const;
 
 	/**
 	 * Queue removal of an analyzer for a given file identifier.
@@ -288,12 +307,7 @@ public:
 	 * @param args a \c AnalyzerArgs value which describes a file analyzer.
 	 * @return true if the analyzer is active at the time of call, else false.
 	 */
-	bool RemoveAnalyzer(const std::string& file_id, const file_analysis::Tag& tag,
-	                    RecordValPtr args) const;
-
-	[[deprecated("Remove in v4.1.  Pass IntrusivePtr args param instead.")]]
-	bool RemoveAnalyzer(const std::string& file_id, const file_analysis::Tag& tag,
-	                    RecordVal* args) const;
+	bool RemoveAnalyzer(const std::string& file_id, const zeek::Tag& tag, RecordValPtr args) const;
 
 	/**
 	 * Tells whether analysis for a file is active or ignored.
@@ -309,11 +323,7 @@ public:
 	 * @param f The file analzer is to be associated with.
 	 * @return The new analyzer instance or null if tag is invalid.
 	 */
-	Analyzer* InstantiateAnalyzer(const Tag& tag, RecordValPtr args,
-	                              File* f) const;
-
-	[[deprecated("Remove in v4.1.  Pass in IntrusivePtr args instead.")]]
-	Analyzer* InstantiateAnalyzer(const Tag& tag, RecordVal* args, File* f) const;
+	Analyzer* InstantiateAnalyzer(const Tag& tag, RecordValPtr args, File* f) const;
 
 	/**
 	 * Returns a set of all matching MIME magic signatures for a given
@@ -326,9 +336,9 @@ public:
 	 * @return Set of all matching file magic signatures, which may be
 	 *         an object allocated by the method if \a rval is a null pointer.
 	 */
-	zeek::detail::RuleMatcher::MIME_Matches* DetectMIME(
-		const u_char* data, uint64_t len,
-		zeek::detail::RuleMatcher::MIME_Matches* rval) const;
+	zeek::detail::RuleMatcher::MIME_Matches*
+	DetectMIME(const u_char* data, uint64_t len,
+	           zeek::detail::RuleMatcher::MIME_Matches* rval) const;
 
 	/**
 	 * Returns the strongest MIME magic signature match for a given data chunk.
@@ -339,14 +349,11 @@ public:
 	 */
 	std::string DetectMIME(const u_char* data, uint64_t len) const;
 
-	uint64_t CurrentFiles()
-		{ return id_map.size(); }
+	uint64_t CurrentFiles() { return id_map.size(); }
 
-	uint64_t MaxFiles()
-		{ return max_files; }
+	uint64_t MaxFiles() { return max_files; }
 
-	uint64_t CumulativeFiles()
-		{ return cumulative_files; }
+	uint64_t CumulativeFiles() { return cumulative_files; }
 
 protected:
 	friend class detail::FileTimer;
@@ -370,15 +377,14 @@ protected:
 	 *         connection-related fields.
 	 */
 	File* GetFile(const std::string& file_id, Connection* conn = nullptr,
-	              const analyzer::Tag& tag = analyzer::Tag::Error,
-	              bool is_orig = false, bool update_conn = true,
-	              const char* source_name = nullptr);
+	              const zeek::Tag& tag = zeek::Tag::Error, bool is_orig = false,
+	              bool update_conn = true, const char* source_name = nullptr);
 
 	/**
 	 * Evaluate timeout policy for a file and remove the File object mapped to
 	 * \a file_id if needed.
 	 * @param file_id the file identifier/hash.
-	 * @param is_termination whether the Manager (and probably Bro) is in a
+	 * @param is_termination whether the Manager (and probably Zeek) is in a
 	 *        terminating state.  If true, then the timeout cannot be postponed.
 	 */
 	void Timeout(const std::string& file_id, bool is_terminating = run_state::terminating);
@@ -401,7 +407,7 @@ protected:
 	 * @return #current_file_id, which is a hash of a unique file handle string
 	 *         set by a \c get_file_handle event handler.
 	 */
-	std::string GetFileID(const analyzer::Tag& tag, Connection* c, bool is_orig);
+	std::string GetFileID(const zeek::Tag& tag, Connection* c, bool is_orig);
 
 	/**
 	 * Check if analysis is available for files transferred over a given
@@ -411,26 +417,26 @@ protected:
 	 * @return whether file analysis is disabled for the analyzer given by
 	 *         \a tag.
 	 */
-	static bool IsDisabled(const analyzer::Tag& tag);
+	static bool IsDisabled(const zeek::Tag& tag);
 
 private:
-	typedef std::set<Tag> TagSet;
-	typedef std::map<std::string, TagSet*> MIMEMap;
+	using TagSet = std::set<Tag>;
+	using MIMEMap = std::map<std::string, TagSet*>;
 
 	TagSet* LookupMIMEType(const std::string& mtype, bool add_if_not_found);
 
-	std::map<std::string, File*> id_map;  /**< Map file ID to file_analysis::File records. */
+	std::map<std::string, File*> id_map; /**< Map file ID to file_analysis::File records. */
 	std::set<std::string> ignored; /**< Ignored files.  Will be finally removed on EOF. */
-	std::string current_file_id;	/**< Hash of what get_file_handle event sets. */
-	zeek::detail::RuleFileMagicState* magic_state;	/**< File magic signature match state. */
-	MIMEMap mime_types;/**< Mapping of MIME types to analyzers. */
+	std::string current_file_id; /**< Hash of what get_file_handle event sets. */
+	zeek::detail::RuleFileMagicState* magic_state; /**< File magic signature match state. */
+	MIMEMap mime_types; /**< Mapping of MIME types to analyzers. */
 
-	inline static TableVal* disabled = nullptr;	/**< Table of disabled analyzers. */
-	inline static TableType* tag_set_type = nullptr;	/**< Type for set[tag]. */
+	inline static TableVal* disabled = nullptr; /**< Table of disabled analyzers. */
+	inline static TableType* tag_set_type = nullptr; /**< Type for set[tag]. */
 
 	size_t cumulative_files;
 	size_t max_files;
-};
+	};
 
 /**
  * Returns a script-layer value corresponding to the \c mime_matches type.
@@ -438,16 +444,8 @@ private:
  */
 VectorValPtr GenMIMEMatchesVal(const zeek::detail::RuleMatcher::MIME_Matches& m);
 
-} // namespace file_analysis
+	} // namespace file_analysis
 
 extern file_analysis::Manager* file_mgr;
 
-} // namespace zeek
-
-namespace file_analysis {
-
-using Manager [[deprecated("Remove in v4.1. Use zeek::file_analysis::Manager.")]] = zeek::file_analysis::Manager;
-
-} // namespace file_analysis
-
-extern zeek::file_analysis::Manager*& file_mgr [[deprecated("Remove in v4.1. Use zeek::file_mgr.")]];
+	} // namespace zeek
