@@ -3,41 +3,46 @@
 #pragma once
 
 #include <sys/types.h>
-#include <iostream>
-#include <vector>
 #include <fstream>
+#include <iostream>
 #include <memory>
+#include <vector>
 
+#include "zeek/Obj.h"
 #include "zeek/input/ReaderBackend.h"
 #include "zeek/threading/formatters/Ascii.h"
 
-namespace zeek::input::reader::detail {
+namespace zeek::input::reader::detail
+	{
 
 // Description for input field mapping.
-struct FieldMapping {
+struct FieldMapping
+	{
 	std::string name;
 	TypeTag type;
 	TypeTag subtype; // internal type for sets and vectors
-	int position;
-	int secondary_position; // for ports: pos of the second field
-	bool present;
+	int position = -1;
+	int secondary_position = -1; // for ports: pos of the second field
+	bool present = false;
 
 	FieldMapping(const std::string& arg_name, const TypeTag& arg_type, int arg_position);
-	FieldMapping(const std::string& arg_name, const TypeTag& arg_type, const TypeTag& arg_subtype, int arg_position);
+	FieldMapping(const std::string& arg_name, const TypeTag& arg_type, const TypeTag& arg_subtype,
+	             int arg_position);
 
 	FieldMapping(const FieldMapping& arg);
-	FieldMapping() { position = -1; secondary_position = -1; }
+	FieldMapping() = default;
 
 	FieldMapping subType();
-};
+	};
 
 /**
  * Reader for structured ASCII files.
  */
-class Ascii : public ReaderBackend {
+class Ascii : public ReaderBackend
+	{
 public:
 	explicit Ascii(ReaderFrontend* frontend);
-	~Ascii() override;
+	~Ascii() override = default;
 
 	// prohibit copying and moving
 	Ascii(const Ascii&) = delete;
@@ -48,11 +53,14 @@ public:
 	static ReaderBackend* Instantiate(ReaderFrontend* frontend) { return new Ascii(frontend); }
 
 protected:
-	bool DoInit(const ReaderInfo& info, int arg_num_fields, const threading::Field* const* fields) override;
+	bool DoInit(const ReaderInfo& info, int arg_num_fields,
+	            const threading::Field* const* fields) override;
 	void DoClose() override;
 	bool DoUpdate() override;
 	bool DoHeartbeat(double network_time, double current_time) override;
 
+	const zeek::detail::Location* GetLocationInfo() const override { return read_location.get(); }
+
 private:
 	bool ReadHeader(bool useCached);
 	bool GetLine(std::string& str);
@@ -83,13 +91,21 @@ private:
 	std::string path_prefix;
 
 	std::unique_ptr<threading::Formatter> formatter;
-};
 
-} // namespace zeek::input::reader::detail
+	// zeek::detail::Location doesn't have a destructor because it's constexpr, so we have to
+	// define a custom deleter for the unique_ptr here to make sure the filename gets deleted
+	// correctly when the unique_ptr gets reset.
+	struct LocationDeleter
+		{
+		void operator()(zeek::detail::Location* loc) const
+			{
+			delete[] loc->filename;
+			delete loc;
+			}
+		};
 
-namespace input::reader {
+	using LocationPtr = std::unique_ptr<zeek::detail::Location, LocationDeleter>;
+	LocationPtr read_location;
+	};
 
-using FieldMapping [[deprecated("Remove in v4.1. Use zeek::input::reader::detail::FieldMapping.")]] = zeek::input::reader::detail::FieldMapping;
-using Ascii [[deprecated("Remove in v4.1. Use zeek::input::reader::detail::Ascii.")]] = zeek::input::reader::detail::Ascii;
-
-} // namespace input::reader
+	} // namespace zeek::input::reader::detail
