@@ -8,22 +8,24 @@
 
 #if defined(__OpenBSD__)
 #include <net/bpf.h>
-typedef struct bpf_timeval pkt_timeval;
+using pkt_timeval = bpf_timeval;
 #else
-typedef struct timeval pkt_timeval;
+using pkt_timeval = struct timeval;
 #endif
 
 #include <pcap.h> // For DLT_ constants
 
+#include "zeek/IP.h"
 #include "zeek/NetVar.h" // For BifEnum::Tunnel
 #include "zeek/TunnelEncapsulation.h"
-#include "zeek/IP.h"
+#include "zeek/session/Session.h"
 
-ZEEK_FORWARD_DECLARE_NAMESPACED(ODesc, zeek);
-ZEEK_FORWARD_DECLARE_NAMESPACED(Val, zeek);
-ZEEK_FORWARD_DECLARE_NAMESPACED(RecordVal, zeek);
+namespace zeek
+	{
 
-namespace zeek {
+class ODesc;
+class Val;
+class RecordVal;
 
 template <class T> class IntrusivePtr;
 using ValPtr = IntrusivePtr<Val>;
@@ -34,17 +36,19 @@ using RecordValPtr = IntrusivePtr<RecordVal>;
  * This enum is sized as an int32_t to make the Packet structure align
  * correctly.
  */
-enum Layer3Proto : int32_t {
-	L3_UNKNOWN = -1,	/// Layer 3 type could not be determined.
-	L3_IPV4 = 1,		/// Layer 3 is IPv4.
-	L3_IPV6 = 2,		/// Layer 3 is IPv6.
-	L3_ARP = 3,			/// Layer 3 is ARP.
-};
+enum Layer3Proto : int32_t
+	{
+	L3_UNKNOWN = -1, /// Layer 3 type could not be determined.
+	L3_IPV4 = 1, /// Layer 3 is IPv4.
+	L3_IPV6 = 2, /// Layer 3 is IPv6.
+	L3_ARP = 3, /// Layer 3 is ARP.
+	};
 
 /**
  * A link-layer packet.
  */
-class Packet {
+class Packet
+	{
 public:
 	/**
 	 * Construct and initialize from packet data.
@@ -68,12 +72,11 @@ public:
 	 * @param tag A textual tag to associate with the packet for
 	 * differentiating the input streams.
 	 */
-	Packet(int link_type, pkt_timeval *ts, uint32_t caplen,
-	       uint32_t len, const u_char *data, bool copy = false,
-	       std::string tag = "")
-	       {
-	       Init(link_type, ts, caplen, len, data, copy, tag);
-	       }
+	Packet(int link_type, pkt_timeval* ts, uint32_t caplen, uint32_t len, const u_char* data,
+	       bool copy = false, std::string tag = "")
+		{
+		Init(link_type, ts, caplen, len, data, copy, tag);
+		}
 
 	/**
 	 * Default constructor. For internal use only.
@@ -111,9 +114,8 @@ public:
 	 * @param tag A textual tag to associate with the packet for
 	 * differentiating the input streams.
 	 */
-	void Init(int link_type, pkt_timeval *ts, uint32_t caplen,
-	          uint32_t len, const u_char *data, bool copy = false,
-	          std::string tag = "");
+	void Init(int link_type, pkt_timeval* ts, uint32_t caplen, uint32_t len, const u_char* data,
+	          bool copy = false, std::string tag = "");
 
 	/**
 	 * Returns a \c raw_pkt_hdr RecordVal, which includes layer 2 and
@@ -121,8 +123,11 @@ public:
 	 */
 	RecordValPtr ToRawPktHdrVal() const;
 
-	[[deprecated("Remove in v4.1.  Use ToRawPktHdrval() instead.")]]
-	RecordVal* BuildPktHdrVal() const;
+	/**
+	 * Returns a RecordVal that represents the Packet. This is used
+	 * by the get_current_packet bif.
+	 */
+	static RecordValPtr ToVal(const Packet* p);
 
 	/**
 	 * Maximal length of a layer 2 address.
@@ -134,16 +139,16 @@ public:
 	 * LinuxSLL packet analyzer doesn't have a destination address in the
 	 * header and thus sets it to this default address.
 	 */
-	static constexpr const u_char L2_EMPTY_ADDR[L2_ADDR_LEN] = { 0 };
+	static constexpr const u_char L2_EMPTY_ADDR[L2_ADDR_LEN] = {0};
 
 	// These are passed in through the constructor.
-	std::string tag;				/// Used in serialization
-	double time;					/// Timestamp reconstituted as float
-	pkt_timeval ts;					/// Capture timestamp
-	const u_char* data = nullptr;	/// Packet data.
-	uint32_t len;					/// Actual length on wire
-	uint32_t cap_len;				/// Captured packet length
-	uint32_t link_type;				/// pcap link_type (DLT_EN10MB, DLT_RAW, etc)
+	std::string tag; /// Used in serialization
+	double time; /// Timestamp reconstituted as float
+	pkt_timeval ts; /// Capture timestamp
+	const u_char* data = nullptr; /// Packet data.
+	uint32_t len; /// Actual length on wire
+	uint32_t cap_len; /// Captured packet length
+	uint32_t link_type; /// pcap link_type (DLT_EN10MB, DLT_RAW, etc)
 
 	/**
 	 * Layer 3 protocol identified (if any).
@@ -168,29 +173,42 @@ public:
 	/**
 	 * (Outermost) VLAN tag if any, else 0.
 	 */
-	uint32_t vlan;
+	uint32_t vlan = 0;
 
 	/**
 	 * (Innermost) VLAN tag if any, else 0.
 	 */
-	uint32_t inner_vlan;
+	uint32_t inner_vlan = 0;
+
+	/**
+	 * If this packet is related to a connection, this flag denotes whether
+	 * this packet is from the originator of the connection.
+	 */
+	bool is_orig = false;
 
 	/**
 	 * Indicates whether the layer 2 checksum was validated by the
 	 * hardware/kernel before being received by zeek.
 	 */
-	bool l2_checksummed;
+	bool l2_checksummed = false;
 
 	/**
 	 * Indicates whether the layer 3 checksum was validated by the
 	 * hardware/kernel before being received by zeek.
 	 */
-	bool l3_checksummed;
+	bool l3_checksummed = false;
 
 	/**
 	 * Indicates whether this packet should be recorded.
 	 */
-	mutable bool dump_packet;
+	mutable bool dump_packet = false;
+
+	/**
+	 * Indicates the amount of data to be dumped. If only a header is needed,
+	 * set this to the size of the header. Setting it to zero will dump the
+	 * entire packet.
+	 */
+	mutable int dump_size = 0;
 
 	// These are fields passed between various packet analyzers. They're best
 	// stored with the packet so they stay available as the packet is passed
@@ -207,7 +225,7 @@ public:
 	 * The IP header for this packet. This is filled in by the IP analyzer
 	 * during processing if the packet contains an IP header.
 	 */
-	std::unique_ptr<IP_Hdr> ip_hdr = nullptr;
+	std::shared_ptr<IP_Hdr> ip_hdr = nullptr;
 
 	/**
 	 * The protocol of the packet. This is used by the tunnel analyzers to
@@ -236,6 +254,19 @@ public:
 	 */
 	int gre_link_type = DLT_RAW;
 
+	/**
+	 * This flag indicates whether a packet has been processed. This can
+	 * mean different things depending on the traffic, but generally it
+	 * means that a packet has been logged in some way. We default to
+	 * false, and this can be set to true for any number of reasons.
+	 */
+	bool processed = false;
+
+	/**
+	 * The session related to this packet, if one exists.
+	 */
+	session::Session* session = nullptr;
+
 private:
 	// Renders an MAC address into its ASCII representation.
 	ValPtr FmtEUI48(const u_char* mac) const;
@@ -243,14 +274,6 @@ private:
 	// True if we need to delete associated packet memory upon
 	// destruction.
 	bool copy;
-};
+	};
 
-} // namespace zeek
-
-using Layer3Proto [[deprecated("Remove in v4.1. Use zeek::Layer3Proto.")]] = zeek::Layer3Proto;
-using Packet [[deprecated("Remove in v4.1. Use zeek::Packet.")]] = zeek::Packet;
-
-constexpr auto L3_UNKNOWN [[deprecated("Remove in v4.1. Use zeek::L3_UNKNOWN")]] = zeek::L3_UNKNOWN;
-constexpr auto L3_IPV4 [[deprecated("Remove in v4.1. Use zeek::L3_IPV4")]] = zeek::L3_IPV4;
-constexpr auto L3_IPV6 [[deprecated("Remove in v4.1. Use zeek::L3_IPV6")]] = zeek::L3_IPV6;
-constexpr auto L3_ARP [[deprecated("Remove in v4.1. Use zeek::L3_ARP")]] = zeek::L3_ARP;
+	} // namespace zeek
