@@ -2,17 +2,20 @@
 
 #pragma once
 
-#include <utility>
 #include <map>
+#include <set>
 #include <string_view>
+#include <utility>
 
+#include "zeek/Reporter.h"
+#include "zeek/ZeekArgs.h"
+#include "zeek/plugin/Component.h"
+#include "zeek/plugin/Plugin.h"
 
-#include "Plugin.h"
-#include "Component.h"
-
-#include "../Reporter.h"
-
-namespace plugin {
+namespace zeek
+	{
+namespace plugin
+	{
 
 // Macros that trigger plugin hooks. We put this into macros to short-cut the
 // code for the most common case that no plugin defines the hook.
@@ -24,8 +27,11 @@ namespace plugin {
  *
  * @param method_call The \a Manager method corresponding to the hook.
  */
-#define PLUGIN_HOOK_VOID(hook, method_call) \
-	{ if ( plugin_mgr->HavePluginForHook(plugin::hook) ) plugin_mgr->method_call; }
+#define PLUGIN_HOOK_VOID(hook, method_call)                                                        \
+		{                                                                                          \
+		if ( zeek::plugin_mgr->HavePluginForHook(zeek::plugin::hook) )                             \
+			zeek::plugin_mgr->method_call;                                                         \
+		}
 
 /**
  * Macro to trigger hooks that return a result.
@@ -37,19 +43,20 @@ namespace plugin {
  * @param default_result: The result to use if there's no plugin implementing
  * the hook.
  */
-#define PLUGIN_HOOK_WITH_RESULT(hook, method_call, default_result) \
-	(plugin_mgr->HavePluginForHook(::plugin::hook) ? plugin_mgr->method_call : (default_result))
+#define PLUGIN_HOOK_WITH_RESULT(hook, method_call, default_result)                                 \
+	(zeek::plugin_mgr->HavePluginForHook(zeek::plugin::hook) ? zeek::plugin_mgr->method_call       \
+	                                                         : (default_result))
 
 /**
  * A singleton object managing all plugins.
  */
 class Manager
-{
+	{
 public:
-	typedef void (*bif_init_func)(Plugin *);
-	typedef std::list<Plugin*> plugin_list;
-	typedef Plugin::component_list component_list;
-	typedef std::list<std::pair<std::string, std::string> > inactive_plugin_list;
+	using bif_init_func = void (*)(Plugin*);
+	using plugin_list = std::list<Plugin*>;
+	using component_list = Plugin::component_list;
+	using inactive_plugin_list = std::list<std::pair<std::string, std::string>>;
 
 	/**
 	 * Constructor.
@@ -79,32 +86,29 @@ public:
 	 * Activating a plugin involves loading its dynamic module, making its
 	 * bifs available, and adding its script paths to ZEEKPATH.
 	 *
+	 * This attempts to activate the plugin immediately. If that fails for
+	 * some reason, we schedule it to be retried later with
+	 * ActivateDynamicPlugins().
+	 *
 	 * @param name The name of the plugin, as found previously by
-	 * SearchPlugin().
-	 *
-	 * @return True if the plugin has been loaded successfully.
-	 *
+	·* SearchPlugin().
 	 */
-	bool ActivateDynamicPlugin(const std::string& name);
+	void ActivateDynamicPlugin(const std::string& name);
 
 	/**
-	 * Activates plugins that SearchDynamicPlugins() has previously discovered.
-	 * The effect is the same all calling \a ActivePlugin(name) for each plugin.
+	 * Activates plugins that SearchDynamicPlugins() has previously discovered,
+	 * including any that have failed to load in prior calls to
+	 * ActivateDynamicPlugin(). Aborts if any plugins fails to activate.
 	 *
 	 * @param all If true, activates all plugins that are found. If false,
 	 * activates only those that should always be activated unconditionally,
-	 * as specified via the ZEEK_PLUGIN_ACTIVATE enviroment variable. In other
-	 * words, it's \c true in standard mode and \c false in bare mode.
-	 *
-	 * @return True if all plugins have been loaded successfully. If one
-	 * fails to load, the method stops there without loading any further ones
-	 * and returns false.
+	 * as specified via the ZEEK_PLUGIN_ACTIVATE environment variable.
 	 */
-	bool ActivateDynamicPlugins(bool all);
+	void ActivateDynamicPlugins(bool all);
 
 	/**
-	 * First-stage initializion of the manager. This is called early on
-	 * during Bro's initialization, before any scripts are processed, and
+	 * First-stage initialization of the manager. This is called early on
+	 * during Zeek's initialization, before any scripts are processed, and
 	 * forwards to the corresponding Plugin methods.
 	 */
 	void InitPreScript();
@@ -117,7 +121,7 @@ public:
 
 	/**
 	 * Third-stage initialization of the manager. This is called late during
-	 * Bro's initialization after any scripts are processed, and forwards to
+	 * Zeek's initialization after any scripts are processed, and forwards to
 	 * the corresponding Plugin methods.
 	 */
 	void InitPostScript();
@@ -148,7 +152,7 @@ public:
 	 * are derived from a specific class. The class is given as the
 	 * template parameter \c T.
 	 */
-	template<class T> std::list<T *> Components() const;
+	template <class T> std::list<T*> Components() const;
 
 	/**
 	 * Returns the (dynamic) plugin associated with a given filesytem
@@ -168,7 +172,7 @@ public:
 	bool HavePluginForHook(HookType hook) const
 		{
 		// Inline to avoid the function call.
-		return hooks[hook] != 0;
+		return hooks[hook] != nullptr;
 		}
 
 	/**
@@ -177,7 +181,7 @@ public:
 	 *
 	 * @param plugin The plugin to return the hooks for.
 	 */
-	std::list<std::pair<HookType, int> > HooksEnabledForPlugin(const Plugin* plugin) const;
+	std::list<std::pair<HookType, int>> HooksEnabledForPlugin(const Plugin* plugin) const;
 
 	/**
 	 * Enables a hook for a given plugin.
@@ -202,8 +206,8 @@ public:
 	/**
 	 * Registers interest in an event by a plugin, even if there's no handler
 	 * for it. Normally a plugin receives events through HookQueueEvent()
-	 * only if Bro actually has code to execute for it. By calling this
-	 * method, the plugin tells Bro to raise the event even if there's no
+	 * only if Zeek actually has code to execute for it. By calling this
+	 * method, the plugin tells Zeek to raise the event even if there's no
 	 * correspondong handler; it will then go into HookQueueEvent() just as
 	 * any other.
 	 *
@@ -214,7 +218,7 @@ public:
 	void RequestEvent(EventHandlerPtr handler, Plugin* plugin);
 
 	/**
-	 * Register interest in the destruction of a BroObj instance. When Bro's
+	 * Register interest in the destruction of a Obj instance. When Zeek's
 	 * reference counting triggers the objects destructor to run, the \a
 	 * HookBroObjDtor will be called.
 	 *
@@ -222,38 +226,68 @@ public:
 	 *
 	 * @param plugin The plugin expressing interest.
 	 */
-	void RequestBroObjDtor(BroObj* obj, Plugin* plugin);
+	void RequestBroObjDtor(Obj* obj, Plugin* plugin);
 
 	// Hook entry functions.
 
 	/**
 	 * Hook that gives plugins a chance to take over loading an input
 	 * file. This method must be called between InitPreScript() and
-	 * InitPostScript() for each input file Bro is about to load, either
+	 * InitPostScript() for each input file Zeek is about to load, either
 	 * given on the command line or via @load script directives. The hook can
-	 * take over the file, in which case Bro must not further process it
+	 * take over the file, in which case Zeek must not further process it
 	 * otherwise.
 	 *
 	 * @return 1 if a plugin took over the file and loaded it successfully; 0
 	 * if a plugin took over the file but had trouble loading it; and -1 if
 	 * no plugin was interested in the file at all.
 	 */
-	virtual int HookLoadFile(const Plugin::LoadType type, const string& file, const string& resolved);
+	virtual int HookLoadFile(const Plugin::LoadType type, const std::string& file,
+	                         const std::string& resolved);
+
+	/**
+	 * Hook that gives plugins a chance to take over loading an input file,
+	 * including replacing the file's content. This method must be called
+	 * between InitPreScript() and InitPostScript() for each input file Zeek is
+	 * about to load, either given on the command line or via @load script
+	 * directives. The hook can take over the file, in which case Zeek must not
+	 * further process it otherwise; or provide its content, in which case Zeek
+	 * must use that and ignore the original file.
+	 *
+	 * @return tuple where the first element is 1 if a plugin took over the
+	 * file; 0 if a plugin took over the file but had trouble loading it; and
+	 * -1 if no plugin was interested in the file at all.
+	 *
+	 * If the plugins takes over by returning 1, there are two cases: if the
+	 * second tuple element remains unset, the plugin handled the loading
+	 * completely internally; the caller must not process it any further.
+	 * Alternatively, the plugin may optionally return the acutal content to
+	 * use for the file as a string through the tuple's second element. If so,
+	 * the caller must ignore the file on disk and use that provided content
+	 * instead (including when there's actually no physical file in place on
+	 * disk at all).
+	 */
+	virtual std::pair<int, std::optional<std::string>>
+	HookLoadFileExtended(const Plugin::LoadType type, const std::string& file,
+	                     const std::string& resolved);
 
 	/**
 	 * Hook that filters calls to a script function/event/hook.
 	 *
 	 * @param func The function to be called.
 	 *
+	 * @param parent The frame in which the function is being called.
+	 *
 	 * @param args The function call's arguments; they may be modified by the
 	 * method.
 	 *
-	 * @return If a plugin handled the call, a Val with a +1 reference count
-	 * containing the result value to pass back to the interpreter (for void
-	 * functions and events, it may be any Val and must be ignored). If no
-	 * plugin handled the call, the method returns null.
+	 * @return If a plugin handled the call, a Val representing the result
+	 * to pass back to the interpreter (for void functions and events,
+	 * it may be any Val and must be ignored). If no plugin handled the call,
+	 * the method returns null.
 	 */
-	std::pair<bool, Val*> HookCallFunction(const Func* func, Frame *parent, val_list* args) const;
+	std::pair<bool, ValPtr> HookCallFunction(const Func* func, zeek::detail::Frame* parent,
+	                                         Args* args) const;
 
 	/**
 	 * Hook that filters the queuing of an event.
@@ -279,7 +313,7 @@ public:
 	 *
 	 * @param conn The connection.
 	 */
-	void HookSetupAnalyzerTree(Connection *conn) const;
+	void HookSetupAnalyzerTree(Connection* conn) const;
 
 	/**
 	 * Hook that informs plugins that the event queue is being drained.
@@ -317,11 +351,8 @@ public:
 	 *
 	 * @param fields threading::Field description of the fields being logged.
 	 */
-	void HookLogInit(const std::string& writer,
-	                 const std::string& instantiating_filter,
-	                 bool local, bool remote,
-	                 const logging::WriterBackend::WriterInfo& info,
-	                 int num_fields,
+	void HookLogInit(const std::string& writer, const std::string& instantiating_filter, bool local,
+	                 bool remote, const logging::WriterBackend::WriterInfo& info, int num_fields,
 	                 const threading::Field* const* fields) const;
 
 	/**
@@ -350,11 +381,9 @@ public:
 	 * @return true if log line should be written, false if log line should be
 	 *         skipped and not passed on to the writer.
 	 */
-	bool HookLogWrite(const std::string& writer,
-	                  const std::string& filter,
-	                  const logging::WriterBackend::WriterInfo& info,
-	                  int num_fields, const threading::Field* const* fields,
-	                  threading::Value** vals) const;
+	bool HookLogWrite(const std::string& writer, const std::string& filter,
+	                  const logging::WriterBackend::WriterInfo& info, int num_fields,
+	                  const threading::Field* const* fields, threading::Value** vals) const;
 
 	/**
 	 * Hook into reporting. This method will be called for each reporter call
@@ -368,7 +397,7 @@ public:
 	 *
 	 * @param conn The associated connection
 	 *
-	 * @param addl Additional Bro values; typically will be passed to the event
+	 * @param addl Additional Zeek values; typically will be passed to the event
 	 *             by the reporter framework.
 	 *
 	 * @param location True if event expects location information
@@ -385,9 +414,19 @@ public:
 	 *         if the event call should be skipped
 	 */
 	bool HookReporter(const std::string& prefix, const EventHandlerPtr event,
-	                  const Connection* conn, const val_list* addl, bool location,
-	                  const Location* location1, const Location* location2,
-	                  bool time, const std::string& message);
+	                  const Connection* conn, const ValPList* addl, bool location,
+	                  const zeek::detail::Location* location1,
+	                  const zeek::detail::Location* location2, bool time,
+	                  const std::string& message);
+
+	/**
+	 * Hook for packets that are considered unprocessed by an Analyzer. This
+	 * typically means that a packet has not had a log entry written for it by
+	 * the time analysis finishes.
+	 *
+	 * @param packet The data for an unprocessed packet
+	 */
+	void HookUnprocessedPacket(const Packet* packet) const;
 
 	/**
 	 * Internal method that registers a freshly instantiated plugin with
@@ -409,26 +448,43 @@ public:
 	 */
 	static void RegisterBifFile(const char* plugin, bif_init_func c);
 
+	/**
+	 * Adds the paths from built-in plugins (e.g. from --include-plugins)
+	 * to ZEEKPATH so they are loaded correctly.
+	 */
+	void ExtendZeekPathForPlugins();
+
 private:
-	bool ActivateDynamicPluginInternal(const std::string& name, bool ok_if_not_found = false);
+	bool ActivateDynamicPluginInternal(const std::string& name, bool ok_if_not_found,
+	                                   std::vector<std::string>* errors);
 	void UpdateInputFiles();
 	void MetaHookPre(HookType hook, const HookArgumentList& args) const;
-	void MetaHookPost(HookType hook, const HookArgumentList& args, HookArgument result) const;
+	void MetaHookPost(HookType hook, const HookArgumentList& args,
+	                  const HookArgument& result) const;
 
-	 // All found dynamic plugins, mapping their names to base directory.
-	typedef std::map<std::string, std::string> dynamic_plugin_map;
+	// Directories that have already been searched for dynamic plugins.
+	// Used to prevent multiple searches of the same dirs (e.g. via symlinks).
+	// The paths stored in the set are made canonical via realpath().
+	std::set<std::string, std::less<>> searched_dirs;
+
+	// Plugins that were explicitly requested to be activated, but failed to
+	// load at first.
+	std::set<std::string> requested_plugins;
+
+	// All found dynamic plugins, mapping their names to base directory.
+	using dynamic_plugin_map = std::map<std::string, std::string>;
 	dynamic_plugin_map dynamic_plugins;
 
 	// We temporarliy buffer scripts to load to get them to load in the
 	// right order.
-	typedef std::list<std::string> file_list;
+	using file_list = std::list<std::string>;
 	file_list scripts_to_load;
 
-	bool init;	// Flag indicating whether InitPreScript() has run yet.
+	bool init; // Flag indicating whether InitPreScript() has run yet.
 
 	// A hook list keeps pairs of plugin and priority interested in a
 	// given hook.
-	typedef std::list<std::pair<int, Plugin*> > hook_list;
+	using hook_list = std::list<std::pair<int, Plugin*>>;
 
 	// An array indexed by HookType. An entry is null if there's no hook
 	// of that type enabled.
@@ -447,27 +503,27 @@ private:
 	// even before the manager exists.
 	static plugin_list* ActivePluginsInternal();
 
-	typedef std::list<bif_init_func> bif_init_func_list;
-	typedef std::map<std::string, bif_init_func_list*> bif_init_func_map;
+	using bif_init_func_list = std::list<bif_init_func>;
+	using bif_init_func_map = std::map<std::string, bif_init_func_list*>;
 
 	// Returns a modifiable map of all bif files. This is a static method
 	// so that plugins can register their bifs even before the manager
 	// exists.
 	static bif_init_func_map* BifFilesInternal();
-};
+	};
 
-template<class T>
-std::list<T *> Manager::Components() const
+template <class T> std::list<T*> Manager::Components() const
 	{
-	std::list<T *> result;
+	std::list<T*> result;
 
-	for ( plugin_list::const_iterator p = ActivePluginsInternal()->begin(); p != ActivePluginsInternal()->end(); p++ )
+	for ( plugin_list::const_iterator p = ActivePluginsInternal()->begin();
+	      p != ActivePluginsInternal()->end(); p++ )
 		{
 		component_list components = (*p)->Components();
 
 		for ( component_list::const_iterator c = components.begin(); c != components.end(); c++ )
 			{
-			T* t = dynamic_cast<T *>(*c);
+			T* t = dynamic_cast<T*>(*c);
 
 			if ( t )
 				result.push_back(t);
@@ -477,20 +533,24 @@ std::list<T *> Manager::Components() const
 	return result;
 	}
 
+namespace detail
+	{
+
 /**
  * Internal class used by bifcl-generated code to register its init functions at runtime.
  */
-class __RegisterBif {
+class __RegisterBif
+	{
 public:
 	__RegisterBif(const char* plugin, Manager::bif_init_func init)
 		{
 		Manager::RegisterBifFile(plugin, init);
 		}
-};
+	};
 
-}
+	} // namespace detail
+	} // namespace plugin
 
-/**
- * The global plugin manager singleton.
- */
 extern plugin::Manager* plugin_mgr;
+
+	} // namespace zeek
