@@ -7,6 +7,7 @@ except ImportError:
 import sys
 import datetime
 import time
+import types
 import ipaddress
 import collections
 
@@ -37,7 +38,7 @@ except:
                 if other.dst(None) is None:
                     return True
 
-                return self.dst(None) == other.dst(None);
+                return self.dst(None) == other.dst(None)
 
             except:
                 return False
@@ -134,7 +135,7 @@ class Subscriber:
             if not msg.is_set():
                 return None
 
-            msg = msg.get();
+            msg = msg.get()
 
         if isinstance(msg, tuple):
             return (msg[0].string(), Data.to_py(msg[1]))
@@ -160,6 +161,36 @@ class Subscriber:
     def remove_topic(self, topic, block=False):
         return self._subscriber.remove_topic(_make_topic(topic), block)
 
+class SafeSubscriber(Subscriber):
+    """Subscriber subclass that makes returnes messages safe to process.
+
+    "Safe" here means safe to Python's type system, particularly regarding
+    hashable types. Broker's data model permits nested complex types, such as
+    sets of tables, but those don't directly work in Python (for example,
+    constructing a set of dicts will complain that dicts aren't hashable). To
+    work around this SafeSubscriber relies on ImmutableData rather than Data
+    (used by regular Subscribers)."""
+
+    def get(self, *args, **kwargs):
+        msg = self._subscriber.get(*args, **kwargs)
+
+        if msg is None:
+            return None
+
+        if isinstance(msg, _broker.OptionalSubscriberBaseValueType):
+            if not msg.is_set():
+                return None
+
+            msg = msg.get()
+
+        if isinstance(msg, tuple):
+            return (msg[0].string(), ImmutableData.to_py(msg[1]))
+
+        if isinstance(msg, _broker.VectorPairTopicData):
+            return [(d[0].string(), ImmutableData.to_py(d[1])) for d in msg]
+
+        assert False
+
 class StatusSubscriber():
     def __init__(self, internal_subscriber):
         self._subscriber = internal_subscriber
@@ -268,6 +299,14 @@ class Store:
         self._parent = None
         self._store = None
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self._store.reset()
+        self._parent = None
+        self._store = None
+
     def name(self):
         return self._store.name()
 
@@ -364,11 +403,28 @@ class Store:
     def _to_expiry(self, e):
         return (_broker.OptionalTimespan(_broker.Timespan(float(e))) if e is not None else _broker.OptionalTimespan())
 
+    def await_idle(self, timeout=None):
+        if timeout:
+            return self._store.await_idle(_broker.Timespan(float(timeout)))
+        else:
+            return self._store.await_idle()
+
+    # Points to the "owning" Endpoint to make sure Python cleans this object up
+    # before destroying the endpoint.
+    _parent = None
+
 class Endpoint(_broker.Endpoint):
-    def make_subscriber(self, topics, qsize = 20):
+    def make_subscriber(self, topics, qsize = 20, subscriber_class=Subscriber):
         topics = _make_topics(topics)
         s = _broker.Endpoint.make_subscriber(self, topics, qsize)
-        return Subscriber(s)
+        return subscriber_class(s)
+
+    def make_safe_subscriber(self, topics, qsize = 20):
+        """A variant of make_subscriber that returns a SafeSubscriber instance. In
+        contrast to the Subscriber class, messages retrieved from
+        SafeSubscribers use immutable, hashable values to ensure Python can
+        represent them. When in doubt, use make_safe_subscriber()."""
+        return self.make_subscriber(topics=topics, qsize=qsize, subscriber_class=SafeSubscriber)
 
     def make_status_subscriber(self, receive_statuses=False):
         s = _broker.Endpoint.make_status_subscriber(self, receive_statuses)
@@ -396,7 +452,6 @@ class Endpoint(_broker.Endpoint):
         bopts = _broker.MapBackendOptions() # Generator expression doesn't work here.
         for (k, v) in opts.items():
             bopts[k] = Data.from_py(v)
-
         s = _broker.Endpoint.attach_master(self, name, type, bopts)
         if not s.is_valid():
             return None
@@ -416,6 +471,12 @@ class Endpoint(_broker.Endpoint):
         result._parent = self
         return result
 
+    def await_peer(self, node, timeout=None):
+        if timeout:
+            return  _broker.Endpoint.await_peer(self, node, _broker.Timespan(float(timeout)))
+        else:
+            return  _broker.Endpoint.await_peer(self, node)
+
     def __enter__(self):
         return self
 
@@ -427,7 +488,6 @@ class Message:
         assert False and "method not overridden"
 
 from . import zeek
-from . import bro
 
 class Data(_broker.Data):
     def __init__(self, x = None):
@@ -473,11 +533,11 @@ class Data(_broker.Data):
             v = _broker.Vector([Data(i) for i in x])
             _broker.Data.__init__(self, v)
 
-        elif isinstance(x, set):
+        elif isinstance(x, set) or isinstance(x, frozenset):
             s = _broker.Set(([Data(i) for i in x]))
             _broker.Data.__init__(self, s)
 
-        elif isinstance(x, dict):
+        elif isinstance(x, dict) or isinstance(x, types.MappingProxyType):
             t = _broker.Table()
             for (k, v) in x.items():
                 t[Data(k)] = Data(v)
@@ -538,61 +598,110 @@ class Data(_broker.Data):
             Data.Type.Timespan: lambda: datetime.timedelta(seconds=d.as_timespan()),
             Data.Type.Timestamp: lambda: datetime.datetime.fromtimestamp(d.as_timestamp(), utc),
             Data.Type.Vector: lambda: to_vector(d.as_vector())
-            }
+        }
 
         try:
             return converters[d.get_type()]()
         except KeyError:
             raise TypeError("unsupported data type: " + str(d.get_type()))
 
+class ImmutableData(Data):
+    """A Data specialization that uses immutable complex types for returned Python
+    objects. For sets, the return type is frozenset, for tables it's a hashable,
+    read-only derivative of dict, and for vectors it's Python tuples.
+    """
+    class HashableReadOnlyDict(dict):
+        def __hash__(self):
+            return hash(frozenset(self.items()))
+
+        def __readonly__(self, *args, **kwargs):
+            raise TypeError('cannot modify this dict')
+
+        # https://stackoverflow.com/a/31049908
+        __setitem__ = __readonly__
+        __delitem__ = __readonly__
+        pop = __readonly__
+        popitem = __readonly__
+        clear = __readonly__
+        update = __readonly__
+        setdefault = __readonly__
+        del __readonly__
+
+    @staticmethod
+    def to_py(d):
+        def to_set(s):
+            return frozenset([ImmutableData.to_py(i) for i in s])
+
+        def to_table(t):
+            tmp = {ImmutableData.to_py(k): ImmutableData.to_py(v) for (k, v) in t.items()}
+            # It's tempting here to use types.MappingProxyType here, but it is
+            # not hashable, so doesn't solve our main problem, and cannot be
+            # derived from.
+            return ImmutableData.HashableReadOnlyDict(tmp.items())
+
+        def to_vector(v):
+            return tuple(ImmutableData.to_py(i) for i in v)
+
+        converters = {
+            Data.Type.Set: lambda: to_set(d.as_set()),
+            Data.Type.Table: lambda: to_table(d.as_table()),
+            Data.Type.Vector: lambda: to_vector(d.as_vector())
+        }
+
+        try:
+            return converters[d.get_type()]()
+        except KeyError:
+            # Fall back on the Data class for types we handle identically.
+            return Data.to_py(d)
+
 ####### TODO: Updated to new Broker API until here.
 
 # # TODO: complete interface
 # class Store:
 #   def __init__(self, handle):
 #     self.store = handle
-# 
+#
 #   def name(self):
 #     return self.store.name()
-# 
+#
 # class Mailbox:
 #   def __init__(self, handle):
 #     self.mailbox = handle
-# 
+#
 #   def descriptor(self):
 #     return self.mailbox.descriptor()
-# 
+#
 #   def empty(self):
 #     return self.mailbox.empty()
-# 
+#
 #   def count(self, n = -1):
 #     return self.mailbox.count(n)
-# 
-# 
+#
+#
 # class Message:
 #   def __init__(self, handle):
 #     self.message = handle
-# 
+#
 #   def topic(self):
 #     return self.message.topic().string()
-# 
+#
 #   def data(self):
 #     return self.message.data() # TODO: unwrap properly
-# 
+#
 #   def __str__(self):
 #     return "%s -> %s" % (self.topic(), str(self.data()))
-# 
-# 
+#
+#
 # class BlockingEndpoint(Endpoint):
 #   def __init__(self, handle):
 #     super(BlockingEndpoint, self).__init__(handle)
-# 
+#
 #   def subscribe(self, topic):
 #     self.endpoint.subscribe(topic)
-# 
+#
 #   def unsubscribe(self, topic):
 #     self.endpoint.unsubscribe(topic)
-# 
+#
 #   def receive(self, x):
 #     if x == Status:
 #       return self.endpoint.receive()
@@ -600,7 +709,7 @@ class Data(_broker.Data):
 #       return Message(self.endpoint.receive())
 #     else:
 #       raise BrokerError("invalid receive type")
-# 
+#
 #   #def receive(self):
 #   #  if fun1 is None:
 #   #    return Message(self.endpoint.receive())
@@ -611,29 +720,29 @@ class Data(_broker.Data):
 #   #      return self.endpoint.receive_msg(fun1)
 #   #    raise BrokerError("invalid receive callback arity; must be 1 or 2")
 #   #  return self.endpoint.receive_msg_or_status(fun1, fun2)
-# 
+#
 #   def mailbox(self):
 #     return Mailbox(self.endpoint.mailbox())
-# 
-# 
+#
+#
 # class NonblockingEndpoint(Endpoint):
 #   def __init__(self, handle):
 #     super(NonblockingEndpoint, self).__init__(handle)
-# 
+#
 #   def subscribe(self, topic, fun):
 #     self.endpoint.subscribe_msg(topic, fun)
-# 
+#
 #   def on_status(fun):
 #     self.endpoint.subscribe_status(fun)
-# 
+#
 #   def unsubscribe(self, topic):
 #     self.endpoint.unsubscribe(topic)
-# 
-# 
+#
+#
 # class Context:
 #   def __init__(self):
 #     self.context = _broker.Context()
-# 
+#
 #   def spawn(self, api):
 #     if api == Blocking:
 #       return BlockingEndpoint(self.context.spawn_blocking())
@@ -641,4 +750,4 @@ class Data(_broker.Data):
 #       return NonblockingEndpoint(self.context.spawn_nonblocking())
 #     else:
 #       raise BrokerError("invalid API flag: " + str(api))
-# 
+#
